print("import: 'schemdraw'")
import schemdraw

print("import: 'schemdraw.elements'")
import schemdraw.elements

print("import: 'schemdraw.logic'")
import schemdraw.logic

print("import: 'schemdraw.dsp'")
import schemdraw.dsp

print("import: 'schemdraw.flow'")
import schemdraw.flow

print("import: 'schemdraw.backends'")
import schemdraw.backends

print("import: 'schemdraw.parsing'")
import schemdraw.parsing

